import { Updater } from "./Particle/Updater";
import { Particles } from "../Options/Classes/Particles/Particles";
import { Shape } from "../Options/Classes/Particles/Shape/Shape";
import { OpacityAnimationStatus, OutMode, RotateDirection, ShapeType, SizeAnimationStatus, StartValueType, } from "../Enums";
import { ColorUtils, Plugins, Utils } from "../Utils";
import { Infecter } from "./Particle/Infecter";
export class Particle {
    constructor(container, position, overrideOptions) {
        var _a, _b, _c, _d, _e, _f, _g, _h, _j, _k;
        this.container = container;
        this.fill = true;
        this.close = true;
        this.links = [];
        this.lastNoiseTime = 0;
        this.destroyed = false;
        const options = container.options;
        const particlesOptions = new Particles();
        particlesOptions.load(options.particles);
        if ((_a = overrideOptions === null || overrideOptions === void 0 ? void 0 : overrideOptions.shape) === null || _a === void 0 ? void 0 : _a.type) {
            const shapeType = overrideOptions.shape.type;
            this.shape = shapeType instanceof Array ? Utils.itemFromArray(shapeType) : shapeType;
            const shapeOptions = new Shape();
            shapeOptions.load(overrideOptions.shape);
            if (this.shape) {
                const shapeData = shapeOptions.options[this.shape];
                if (shapeData) {
                    this.shapeData = Utils.deepExtend({}, shapeData instanceof Array ? Utils.itemFromArray(shapeData) : shapeData);
                }
            }
        }
        else {
            const shapeType = particlesOptions.shape.type;
            this.shape = shapeType instanceof Array ? Utils.itemFromArray(shapeType) : shapeType;
            const shapeData = particlesOptions.shape.options[this.shape];
            if (shapeData) {
                this.shapeData = Utils.deepExtend({}, shapeData instanceof Array ? Utils.itemFromArray(shapeData) : shapeData);
            }
        }
        if (overrideOptions !== undefined) {
            particlesOptions.load(overrideOptions);
        }
        if (((_b = this.shapeData) === null || _b === void 0 ? void 0 : _b.particles) !== undefined) {
            particlesOptions.load((_c = this.shapeData) === null || _c === void 0 ? void 0 : _c.particles);
        }
        this.fill = (_e = (_d = this.shapeData) === null || _d === void 0 ? void 0 : _d.fill) !== null && _e !== void 0 ? _e : this.fill;
        this.close = (_g = (_f = this.shapeData) === null || _f === void 0 ? void 0 : _f.close) !== null && _g !== void 0 ? _g : this.close;
        this.particlesOptions = particlesOptions;
        const noiseDelay = this.particlesOptions.move.noise.delay;
        this.noiseDelay =
            (noiseDelay.random.enable
                ? Utils.randomInRange(noiseDelay.random.minimumValue, noiseDelay.value)
                : noiseDelay.value) * 1000;
        container.retina.initParticle(this);
        const color = this.particlesOptions.color;
        const sizeValue = (_h = this.sizeValue) !== null && _h !== void 0 ? _h : container.retina.sizeValue;
        const randomSize = typeof this.particlesOptions.size.random === "boolean"
            ? this.particlesOptions.size.random
            : this.particlesOptions.size.random.enable;
        this.size = {
            value: randomSize && this.randomMinimumSize !== undefined
                ? Utils.randomInRange(this.randomMinimumSize, sizeValue)
                : sizeValue,
        };
        this.direction = this.particlesOptions.move.direction;
        this.bubble = {
            inRange: false,
        };
        this.initialVelocity = this.calculateVelocity();
        this.velocity = {
            horizontal: this.initialVelocity.horizontal,
            vertical: this.initialVelocity.vertical,
        };
        const rotateOptions = this.particlesOptions.rotate;
        const initialAngle = rotateOptions.path
            ? Math.atan2(this.initialVelocity.vertical, this.initialVelocity.horizontal)
            : 0;
        const degAngle = rotateOptions.random ? Math.random() * 360 : rotateOptions.value;
        this.angle = initialAngle + (degAngle * Math.PI) / 180;
        this.rotateDirection = rotateOptions.direction;
        if (this.rotateDirection === RotateDirection.random) {
            const index = Math.floor(Math.random() * 2);
            this.rotateDirection = index > 0 ? RotateDirection.counterClockwise : RotateDirection.clockwise;
        }
        const sizeAnimation = this.particlesOptions.size.animation;
        if (sizeAnimation.enable) {
            switch (sizeAnimation.startValue) {
                case StartValueType.min:
                    if (!randomSize) {
                        const pxRatio = container.retina.pixelRatio;
                        this.size.value = sizeAnimation.minimumValue * pxRatio;
                    }
                    break;
            }
            this.size.status = SizeAnimationStatus.increasing;
            this.size.velocity = ((_j = this.sizeAnimationSpeed) !== null && _j !== void 0 ? _j : container.retina.sizeAnimationSpeed) / 100;
            if (!sizeAnimation.sync) {
                this.size.velocity *= Math.random();
            }
        }
        this.color = ColorUtils.colorToHsl(color);
        const colorAnimation = this.particlesOptions.color.animation;
        if (colorAnimation.enable) {
            this.colorVelocity = colorAnimation.speed / 100;
            if (!colorAnimation.sync) {
                this.colorVelocity = this.colorVelocity * Math.random();
            }
        }
        else {
            this.colorVelocity = 0;
        }
        if (colorAnimation.enable && !colorAnimation.sync && this.color) {
            this.color.h = Math.random() * 360;
        }
        this.position = this.calcPosition(this.container, position);
        this.offset = {
            x: 0,
            y: 0,
        };
        if (this.particlesOptions.collisions.enable && !this.checkOverlap(position)) {
            throw new Error();
        }
        const opacityOptions = this.particlesOptions.opacity;
        const randomOpacity = opacityOptions.random;
        const opacityValue = opacityOptions.value;
        this.opacity = {
            value: randomOpacity.enable ? Utils.randomInRange(randomOpacity.minimumValue, opacityValue) : opacityValue,
        };
        const opacityAnimation = opacityOptions.animation;
        if (opacityAnimation.enable) {
            this.opacity.status = OpacityAnimationStatus.increasing;
            this.opacity.velocity = opacityAnimation.speed / 100;
            if (!opacityAnimation.sync) {
                this.opacity.velocity *= Math.random();
            }
        }
        let drawer = container.drawers.get(this.shape);
        if (!drawer) {
            drawer = Plugins.getShapeDrawer(this.shape);
            if (drawer) {
                container.drawers.set(this.shape, drawer);
            }
        }
        this.loadImageShape(container, drawer);
        this.stroke =
            this.particlesOptions.stroke instanceof Array
                ? Utils.itemFromArray(this.particlesOptions.stroke)
                : this.particlesOptions.stroke;
        this.strokeColor = ColorUtils.colorToHsl(this.stroke.color);
        if (typeof this.stroke.color !== "string") {
            const strokeColorAnimation = (_k = this.stroke.color) === null || _k === void 0 ? void 0 : _k.animation;
            if (strokeColorAnimation && this.strokeColor) {
                if (strokeColorAnimation.enable) {
                    this.strokeColorVelocity = colorAnimation.speed / 100;
                    if (!strokeColorAnimation.sync) {
                        this.strokeColorVelocity = this.strokeColorVelocity * Math.random();
                    }
                }
                else {
                    this.strokeColorVelocity = 0;
                }
                if (strokeColorAnimation.enable && !strokeColorAnimation.sync && this.color) {
                    this.strokeColor.h = Math.random() * 360;
                }
            }
        }
        this.shadowColor = ColorUtils.colorToRgb(this.particlesOptions.shadow.color);
        this.updater = new Updater(this.container, this);
        this.infecter = new Infecter(this.container, this);
    }
    update(delta) {
        this.updater.update(delta);
    }
    draw(delta) {
        this.container.canvas.drawParticle(this, delta);
    }
    isOverlapping() {
        const container = this.container;
        let collisionFound = false;
        const pos1 = this.getPosition();
        for (const p2 of container.particles.array.filter((t) => t != this)) {
            const pos2 = p2.getPosition();
            const dist = Utils.getDistance(pos1, pos2);
            if (dist <= this.size.value + p2.size.value) {
                collisionFound = true;
                break;
            }
        }
        return collisionFound;
    }
    getPosition() {
        return {
            x: this.position.x + this.offset.x,
            y: this.position.y + this.offset.y,
        };
    }
    getFillColor() {
        var _a;
        return (_a = this.bubble.color) !== null && _a !== void 0 ? _a : this.color;
    }
    getStrokeColor() {
        var _a, _b;
        return (_b = (_a = this.bubble.color) !== null && _a !== void 0 ? _a : this.strokeColor) !== null && _b !== void 0 ? _b : this.color;
    }
    destroy() {
        this.destroyed = true;
    }
    checkOverlap(position, iterations = 0) {
        const container = this.container;
        if (!container.particles.count) {
            return true;
        }
        if (iterations >= container.particles.count) {
            return false;
        }
        const overlapping = this.isOverlapping();
        console.log(overlapping);
        if (overlapping) {
            this.position.x = position ? position.x : Math.random() * container.canvas.size.width;
            this.position.y = position ? position.y : Math.random() * container.canvas.size.height;
            return this.checkOverlap(undefined, iterations + 1);
        }
        return true;
    }
    calcPosition(container, position) {
        var _a, _b;
        for (const [, plugin] of container.plugins) {
            const pluginPos = plugin.particlePosition !== undefined ? plugin.particlePosition(position, this) : undefined;
            if (pluginPos !== undefined) {
                return Utils.deepExtend({}, pluginPos);
            }
        }
        const pos = {
            x: (_a = position === null || position === void 0 ? void 0 : position.x) !== null && _a !== void 0 ? _a : Math.random() * container.canvas.size.width,
            y: (_b = position === null || position === void 0 ? void 0 : position.y) !== null && _b !== void 0 ? _b : Math.random() * container.canvas.size.height,
        };
        const outMode = this.particlesOptions.move.outMode;
        if (Utils.isInArray(outMode, OutMode.bounce) || Utils.isInArray(outMode, OutMode.bounceHorizontal)) {
            if (pos.x > container.canvas.size.width - this.size.value * 2) {
                pos.x -= this.size.value;
            }
            else if (pos.x < this.size.value * 2) {
                pos.x += this.size.value;
            }
        }
        if (Utils.isInArray(outMode, OutMode.bounce) || Utils.isInArray(outMode, OutMode.bounceVertical)) {
            if (pos.y > container.canvas.size.height - this.size.value * 2) {
                pos.y -= this.size.value;
            }
            else if (pos.y < this.size.value * 2) {
                pos.y += this.size.value;
            }
        }
        return pos;
    }
    calculateVelocity() {
        const baseVelocity = Utils.getParticleBaseVelocity(this);
        const res = {
            horizontal: 0,
            vertical: 0,
        };
        const moveOptions = this.particlesOptions.move;
        const rad = (Math.PI / 180) * moveOptions.angle;
        const rad45 = Math.PI / 4;
        const range = {
            left: Math.sin(rad45 + rad / 2) - Math.sin(rad45 - rad / 2),
            right: Math.cos(rad45 + rad / 2) - Math.cos(rad45 - rad / 2),
        };
        if (moveOptions.straight) {
            res.horizontal = baseVelocity.x;
            res.vertical = baseVelocity.y;
            if (moveOptions.random) {
                res.horizontal += Utils.randomInRange(range.left, range.right) / 2;
                res.vertical += Utils.randomInRange(range.left, range.right) / 2;
            }
        }
        else {
            res.horizontal = baseVelocity.x + Utils.randomInRange(range.left, range.right) / 2;
            res.vertical = baseVelocity.y + Utils.randomInRange(range.left, range.right) / 2;
        }
        return res;
    }
    loadImageShape(container, drawer) {
        var _a, _b, _c, _d;
        if (!(this.shape === ShapeType.image || this.shape === ShapeType.images)) {
            return;
        }
        const shape = this.particlesOptions.shape;
        const imageDrawer = drawer;
        const imagesOptions = shape.options[this.shape];
        const images = imageDrawer.getImages(container).images;
        const image = Utils.itemFromArray(images);
        const optionsImage = (imagesOptions instanceof Array
            ? imagesOptions.find((t) => t.src === image.source)
            : imagesOptions);
        const color = this.getFillColor();
        let imageRes;
        if ((image === null || image === void 0 ? void 0 : image.svgData) !== undefined && optionsImage.replaceColor && color) {
            const svgColoredData = Utils.replaceColorSvg(image, color, this.opacity.value);
            const svg = new Blob([svgColoredData], { type: "image/svg+xml" });
            const domUrl = window.URL || window.webkitURL || window;
            const url = domUrl.createObjectURL(svg);
            const img = new Image();
            imageRes = {
                data: image,
                loaded: false,
                ratio: optionsImage.width / optionsImage.height,
                replaceColor: (_a = optionsImage.replaceColor) !== null && _a !== void 0 ? _a : optionsImage.replace_color,
                source: optionsImage.src,
            };
            img.addEventListener("load", () => {
                if (this.image) {
                    this.image.loaded = true;
                    image.element = img;
                }
                domUrl.revokeObjectURL(url);
            });
            img.addEventListener("error", () => {
                domUrl.revokeObjectURL(url);
                Utils.loadImage(optionsImage.src).then((img2) => {
                    if (this.image) {
                        image.element = img2.element;
                        this.image.loaded = true;
                    }
                });
            });
            img.src = url;
        }
        else {
            imageRes = {
                data: image,
                loaded: true,
                ratio: optionsImage.width / optionsImage.height,
                replaceColor: (_b = optionsImage.replaceColor) !== null && _b !== void 0 ? _b : optionsImage.replace_color,
                source: optionsImage.src,
            };
        }
        if (!imageRes.ratio) {
            imageRes.ratio = 1;
        }
        const fill = (_c = optionsImage.fill) !== null && _c !== void 0 ? _c : this.fill;
        const close = (_d = optionsImage.close) !== null && _d !== void 0 ? _d : this.close;
        return {
            image: imageRes,
            fill,
            close,
        };
    }
}
