import { Grabber } from "./Interactions/Mouse/Grabber";
import { Repulser } from "./Interactions/Mouse/Repulser";
import { Bubbler } from "./Interactions/Mouse/Bubbler";
import { Connector } from "./Interactions/Mouse/Connector";
import { Linker } from "./Interactions/Particles/Linker";
import { Attractor as ParticlesAttractor } from "./Interactions/Particles/Attractor";
import { Collider } from "./Interactions/Particles/Collider";
import { Infecter } from "./Interactions/Particles/Infecter";
import { TrailMaker } from "./Interactions/Mouse/TrailMaker";
import { Attractor as MouseAttractor } from "./Interactions/Mouse/Attractor";
export class InteractionManager {
    constructor(container) {
        this.container = container;
        this.externalInteractors = [
            new MouseAttractor(container),
            new Bubbler(container),
            new Connector(container),
            new Grabber(container),
            new Repulser(container),
            new TrailMaker(container),
        ];
        this.particleInteractors = [
            new ParticlesAttractor(container),
            new Collider(container),
            new Infecter(container),
            new Linker(container),
        ];
    }
    init() {
    }
    interact(delta) {
        this.externalInteract(delta);
        this.particlesInteract(delta);
    }
    externalInteract(delta) {
        for (const interactor of this.externalInteractors) {
            if (interactor.isEnabled()) {
                interactor.interact(delta);
            }
        }
    }
    particlesInteract(delta) {
        for (const particle of this.container.particles.array) {
            for (const interactor of this.externalInteractors) {
                interactor.reset(particle);
            }
            for (const interactor of this.particleInteractors) {
                if (interactor.isEnabled(particle)) {
                    interactor.interact(particle, delta);
                }
            }
        }
    }
}
