import { Constants, Utils, Circle, ColorUtils } from "../../../../Utils";
import { HoverMode } from "../../../../Enums/Modes";
export class Grabber {
    constructor(container) {
        this.container = container;
    }
    isEnabled() {
        const container = this.container;
        const mouse = container.interactivity.mouse;
        const events = container.options.interactivity.events;
        if (!(events.onHover.enable && mouse.position)) {
            return false;
        }
        const hoverMode = events.onHover.mode;
        return Utils.isInArray(HoverMode.grab, hoverMode);
    }
    reset() {
    }
    interact() {
        var _a, _b;
        const container = this.container;
        const options = container.options;
        const interactivity = options.interactivity;
        if (interactivity.events.onHover.enable && container.interactivity.status === Constants.mouseMoveEvent) {
            const mousePos = container.interactivity.mouse.position;
            if (mousePos === undefined) {
                return;
            }
            const distance = container.retina.grabModeDistance;
            const query = container.particles.quadTree.query(new Circle(mousePos.x, mousePos.y, distance));
            for (const particle of query) {
                const pos = particle.getPosition();
                const distance = Utils.getDistance(pos, mousePos);
                if (distance <= container.retina.grabModeDistance) {
                    const grabLineOptions = interactivity.modes.grab.links;
                    const lineOpacity = grabLineOptions.opacity;
                    const grabDistance = container.retina.grabModeDistance;
                    const opacityLine = lineOpacity - (distance * lineOpacity) / grabDistance;
                    if (opacityLine > 0) {
                        const optColor = (_a = grabLineOptions.color) !== null && _a !== void 0 ? _a : particle.particlesOptions.links.color;
                        if (!container.particles.grabLineColor) {
                            container.particles.grabLineColor =
                                optColor === Constants.randomColorValue ||
                                    ((_b = optColor) === null || _b === void 0 ? void 0 : _b.value) === Constants.randomColorValue
                                    ? Constants.randomColorValue
                                    : ColorUtils.colorToRgb(optColor);
                        }
                        let colorLine;
                        if (container.particles.grabLineColor === Constants.randomColorValue) {
                            colorLine = ColorUtils.getRandomRgbColor();
                        }
                        else {
                            colorLine = container.particles.grabLineColor;
                        }
                        if (colorLine === undefined) {
                            return;
                        }
                        container.canvas.drawGrabLine(particle, colorLine, opacityLine, mousePos);
                    }
                }
            }
        }
    }
}
