import { Interactivity } from "./Interactivity/Interactivity";
import { Particles } from "./Particles/Particles";
import { BackgroundMask } from "./BackgroundMask/BackgroundMask";
import { Background } from "./Background/Background";
import { Infection } from "./Infection/Infection";
import { Plugins } from "../../Utils";
export class Options {
    constructor() {
        this.background = new Background();
        this.backgroundMask = new BackgroundMask();
        this.detectRetina = true;
        this.fpsLimit = 30;
        this.infection = new Infection();
        this.interactivity = new Interactivity();
        this.particles = new Particles();
        this.pauseOnBlur = true;
    }
    get fps_limit() {
        return this.fpsLimit;
    }
    set fps_limit(value) {
        this.fpsLimit = value;
    }
    get retina_detect() {
        return this.detectRetina;
    }
    set retina_detect(value) {
        this.detectRetina = value;
    }
    load(data) {
        var _a, _b;
        if (data === undefined) {
            return;
        }
        if (data.preset !== undefined) {
            if (data.preset instanceof Array) {
                for (const preset of data.preset) {
                    this.importPreset(preset);
                }
            }
            else {
                this.importPreset(data.preset);
            }
        }
        const detectRetina = (_a = data.detectRetina) !== null && _a !== void 0 ? _a : data.retina_detect;
        if (detectRetina !== undefined) {
            this.detectRetina = detectRetina;
        }
        const fpsLimit = (_b = data.fpsLimit) !== null && _b !== void 0 ? _b : data.fps_limit;
        if (fpsLimit !== undefined) {
            this.fpsLimit = fpsLimit;
        }
        if (data.pauseOnBlur !== undefined) {
            this.pauseOnBlur = data.pauseOnBlur;
        }
        this.background.load(data.background);
        this.particles.load(data.particles);
        this.infection.load(data.infection);
        this.interactivity.load(data.interactivity);
        this.backgroundMask.load(data.backgroundMask);
        Plugins.loadOptions(this, data);
    }
    importPreset(preset) {
        this.load(Plugins.getPreset(preset));
    }
}
